<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - ARMCI_Malloc_F.f90.xsl: creates ARMCI_Malloc/Free procedures.
  -
  - Output should be directed to the file, ARMCI_Malloc_F.f90
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

  <xsl:variable name="comment-header">
    <xsl:text>/** ------------------------------------------------------------------- */</xsl:text>
  </xsl:variable>


 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">
    <xsl:call-template name="openFile"/>

<xsl:text>#if defined(F90_SYM_CASE_LOWER)
</xsl:text>

    <xsl:for-each select="type">
      <xsl:call-template name="defineSymLower">
        <xsl:with-param name="type" select="@name"/>
      </xsl:call-template>
    </xsl:for-each>

<xsl:text>#endif /* F90_SYM_CASE_LOWER */

#if defined(F90_SYM_CASE_MIXED)
</xsl:text>

    <xsl:for-each select="type">
      <xsl:call-template name="defineSymMixed">
        <xsl:with-param name="type" select="@name"/>
      </xsl:call-template>
    </xsl:for-each>

<xsl:text>#endif /* F90_SYM_CASE_MIXED */

#if defined(F90_SYM_CASE_UPPER)
</xsl:text>

    <xsl:for-each select="type">
      <xsl:call-template name="defineSymUpper">
        <xsl:with-param name="type" select="@name"/>
      </xsl:call-template>
    </xsl:for-each>

<xsl:text>#endif /* F90_SYM_CASE_UPPER */


</xsl:text>

    <xsl:for-each select="type">

      <xsl:call-template name="defineMalloc">
        <xsl:with-param name="type" select="@name"/>
      </xsl:call-template>

      <xsl:call-template name="defineFree">
        <xsl:with-param name="type" select="@name"/>
      </xsl:call-template>

    </xsl:for-each>

    <xsl:call-template name="closeFile"/>
  </xsl:template>


  <!--
    - defineSymLower
    -->
  <xsl:template name="defineSymLower">
    <xsl:param name="type"/>
    <xsl:variable name="mall_n" select="concat('ARMCI_Malloc_', $type)"/>
    <xsl:variable name="free_n" select="concat('ARMCI_Free_', $type)"/>

    <xsl:variable name="mall_nl">
      <xsl:call-template name="lower-case">
        <xsl:with-param name="symbol" select="$mall_n"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="free_nl">
      <xsl:call-template name="lower-case">
        <xsl:with-param name="symbol" select="$free_n"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:text>#  define </xsl:text> <xsl:value-of select="$mall_n"/>
        <xsl:text> F90_SYMBOL( </xsl:text> <xsl:value-of select="$mall_nl"/>
        <xsl:text> )</xsl:text> <xsl:value-of select="$newline"/>

    <xsl:text>#  define </xsl:text> <xsl:value-of select="$free_n"/>
        <xsl:text> F90_SYMBOL( </xsl:text> <xsl:value-of select="$free_nl"/>
        <xsl:text> )</xsl:text> <xsl:value-of select="$newline"/>

  </xsl:template>


  <!--
    - defineSymMixed
    -->
  <xsl:template name="defineSymMixed">
    <xsl:param name="type"/>
    <xsl:variable name="mall_n" select="concat('ARMCI_Malloc_', $type)"/>
    <xsl:variable name="free_n" select="concat('ARMCI_Free_', $type)"/>

    <xsl:text>#  define </xsl:text> <xsl:value-of select="$mall_n"/>
        <xsl:text> F90_SYMBOL( </xsl:text> <xsl:value-of select="$mall_n"/>
        <xsl:text> )</xsl:text> <xsl:value-of select="$newline"/>

    <xsl:text>#  define </xsl:text> <xsl:value-of select="$free_n"/>
        <xsl:text> F90_SYMBOL( </xsl:text> <xsl:value-of select="$free_n"/>
        <xsl:text> )</xsl:text> <xsl:value-of select="$newline"/>

  </xsl:template>


  <!--
    - defineSymUpper
    -->
  <xsl:template name="defineSymUpper">
    <xsl:param name="type"/>
    <xsl:variable name="mall_n" select="concat('ARMCI_Malloc_', $type)"/>
    <xsl:variable name="free_n" select="concat('ARMCI_Free_', $type)"/>

    <xsl:variable name="mall_nu">
      <xsl:call-template name="upper-case">
        <xsl:with-param name="symbol" select="$mall_n"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="free_nu">
      <xsl:call-template name="upper-case">
        <xsl:with-param name="symbol" select="$free_n"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:text>#  define </xsl:text> <xsl:value-of select="$mall_n"/>
        <xsl:text> F90_SYMBOL( </xsl:text> <xsl:value-of select="$mall_nu"/>
        <xsl:text> )</xsl:text> <xsl:value-of select="$newline"/>

    <xsl:text>#  define </xsl:text> <xsl:value-of select="$free_n"/>
        <xsl:text> F90_SYMBOL( </xsl:text> <xsl:value-of select="$free_nu"/>
        <xsl:text> )</xsl:text> <xsl:value-of select="$newline"/>

  </xsl:template>


  <!--
    - defineMalloc
    -->
  <xsl:template name="defineMalloc">
    <xsl:param name="type"/>
    <xsl:variable name="name" select="concat('ARMCI_Malloc_', $type)"/>

    <xsl:value-of select="concat($comment-header, $newline)"/>
<xsl:text>void </xsl:text> <xsl:value-of select="$name"/>
<xsl:text>(void* dv, int* rank, int lb_in[], int ub_in[], int* rc)
{
  F90_CompilerCharacteristics cc;
  int i;
  void* data;

  long lb[7], stride[7];
  unsigned long extent[7];
  index_size_t lb_armci[7], ub_armci[7];
  unsigned long elem_size = sizeof(</xsl:text>
                            <xsl:value-of select="@ctype"/> <xsl:text>);

  F90_SetCompilerCharacteristics(&amp;cc, FORTRAN_COMPILER);

  lb[0] = lb_in[0];
  extent[0] = 1 + ub_in[0] - lb_in[0];
  stride[0] = elem_size;
  lb_armci[0] = lb_in[0];
  ub_armci[0] = ub_in[0];

  for (i = 1; i &lt; *rank; i++) {
    lb[i] = lb_in[1];
    extent[i] = 1 + ub_in[i] - lb_in[i];
    stride[i] = extent[i-1] * stride[i-1];
    lb_armci[i] = lb_in[i];
    ub_armci[i] = ub_in[i];
  }

  /*
   * Create array memory
   */

   if ( ! armci_init_loc ) {
     ARMCI_Init();
     armci_init_loc = 1;
   }

  _array_create(&amp;data, elem_size, *rank, lb_armci, ub_armci);
  assert(data);

  *rc = cc.resetArrayDesc(dv, data, *rank, lb, extent, stride);
  if (*rc) {
    fprintf(stderr, "ERROR in resetting array descriptor\n");
    return;
  }
}

</xsl:text>

  </xsl:template>


  <!--
    - defineFree
    -->
  <xsl:template name="defineFree">
    <xsl:param name="type"/>
    <xsl:variable name="name" select="concat('ARMCI_Free_', $type)"/>

    <xsl:value-of select="concat($comment-header, $newline)"/>
<xsl:text>void </xsl:text> <xsl:value-of select="$name"/>
<xsl:text>(void* dv, int* rank, int* rc)
{
  F90_CompilerCharacteristics cc;
  void* data;

  F90_SetCompilerCharacteristics(&amp;cc, FORTRAN_COMPILER);

  /*
   * Free array memory
   */
  data = cc.getArrayBaseAddress(dv, *rank);
  if (data == 0) {
    fprintf(stderr, "ERROR in getArrayBaseAddress\n");
    *rc = -1;
    return;
  }
  _array_free(data);   /* ARMCI call to free memory */
}

</xsl:text>

  </xsl:template>


  <!--
    - openFile
    -->
  <xsl:template name="openFile">
<xsl:text>/** LANL:license
 * -------------------------------------------------------------------------
 * This SOFTWARE has been authored by an employee or employees of the
 * University of California, operator of the Los Alamos National Laboratory
 * under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 * The U.S. Government has rights to use, reproduce, and distribute this
 * SOFTWARE.  The public may copy, distribute, prepare derivative works and
 * publicly display this SOFTWARE without charge, provided that this Notice
 * and any statement of authorship are reproduced on all copies.  Neither
 * the Government nor the University makes any warranty, express or implied,
 * or assumes any liability or responsibility for the use of this SOFTWARE.
 * If SOFTWARE is modified to produce derivative works, such modified
 * SOFTWARE should be clearly marked, so as not to confuse it with the
 * version available from LANL.
 * -------------------------------------------------------------------------
 * LANL:license
 * -------------------------------------------------------------------------
 *
 * ARMCI_Malloc_C.c - this file is automatically created
 *
 * The C procedures in this file allocate and free ARMCI based arrays and
 * are designed to be called from Fortran.
 *
 * -------------------------------------------------------------------------
 */

#include "CompilerCharacteristics.h"
#include &lt;F90Compiler.h>
#include &lt;armci.h>
#include &lt;arraydesc.h>
#include &lt;stdio.h>
#include &lt;stdlib.h>
#include &lt;assert.h>

typedef struct {
  float real;
  float imag;
} complex_t;

typedef struct {
  double real;
  double imag;
} dcomplex_t;

int armci_init_loc = 0;

</xsl:text>

  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile"/>


<!--
  - lower-case
  -->
<xsl:template name="lower-case">
  <xsl:param name="symbol" select="unknown_symbol"/>
  <xsl:value-of select="translate($symbol,
                           'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
                           'abcdefghijklmnopqrstuvwxyz')"/>
</xsl:template>


<!--
  - upper-case
  -->
<xsl:template name="upper-case">
  <xsl:param name="symbol" select="unknown_symbol"/>
  <xsl:value-of select="translate($symbol,
                           'abcdefghijklmnopqrstuvwxyz',
                           'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
</xsl:template>


</xsl:stylesheet>
