<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - ARMCI_Malloc_F.f90.xsl: creates ARMCI_Malloc/Free procedures.
  -
  - Output should be directed to the file, ARMCI_Malloc_F.f90
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

  <xsl:variable name="comment-header">
    <xsl:text>!--------------------------------------------------------------------------</xsl:text>
  </xsl:variable>


 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">
    <xsl:call-template name="openFile"/>
    <xsl:call-template name="openArrayModule"/>

    <xsl:value-of select="$newline"/>
    <xsl:text>  interface ARMCI_Malloc</xsl:text>

    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="defineMallocInterface">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>

    <xsl:value-of select="$newline"/>
    <xsl:text>  end interface</xsl:text>
    <xsl:value-of select="$newline"/>

    <xsl:value-of select="$newline"/>
    <xsl:text>  interface ARMCI_Free</xsl:text>

    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="defineFreeInterface">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>

    <xsl:value-of select="$newline"/>
    <xsl:text>  end interface</xsl:text>

<xsl:text>

contains

</xsl:text>

    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">

        <xsl:call-template name="defineMalloc">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>

        <xsl:call-template name="defineFree">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>

      </xsl:for-each>
    </xsl:for-each>

    <xsl:call-template name="closeArrayModule"/>

    <xsl:call-template name="closeFile"/>
  </xsl:template>


  <!--
    - defineMallocInterface
    -->
  <xsl:template name="defineMallocInterface">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="name" select="concat($rank, 'D', $type)"/>

    <xsl:text>
    module procedure ARMCI_Malloc_</xsl:text> <xsl:value-of select="$name"/>

  </xsl:template>


  <!--
    - defineFreeInterface
    -->
  <xsl:template name="defineFreeInterface">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="name" select="concat($rank, 'D', $type)"/>

    <xsl:text>
    module procedure ARMCI_Free_</xsl:text> <xsl:value-of select="$name"/>

  </xsl:template>


  <!--
    - defineMalloc
    -->
  <xsl:template name="defineMalloc">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="dim" select="concat($rank, 'D')"/>
    <xsl:variable name="ext" select="concat($rank, 'D', $type)"/>

    <xsl:value-of select="$comment-header"/>

<xsl:text>
subroutine ARMCI_Malloc_</xsl:text>
           <xsl:value-of select="$ext"/> <xsl:text>(a, lb, ub, rc)
  use F90_kinds
  implicit none
  interface
    subroutine ARMCI_Malloc_</xsl:text>
               <xsl:value-of select="$type"/> <xsl:text>(a, rank, lb, ub, rc)
      use F90_kinds
      </xsl:text> <xsl:value-of select="concat(@ftype, @kind)"/>
                  <xsl:text>, pointer :: a(</xsl:text>
                  <xsl:call-template name="add-dimensions">
                    <xsl:with-param name="rank" select="$rank"/>
                  </xsl:call-template>
                  <xsl:text>)
      integer, intent(in)  :: rank, lb(</xsl:text>
          <xsl:value-of select="concat($rank, '), ub(', $rank)"/> <xsl:text>)
      integer, intent(out) :: rc
    end subroutine ARMCI_Malloc_</xsl:text>
                  <xsl:value-of select="$type"/> <xsl:text>
  end interface
  </xsl:text> <xsl:value-of select="concat(@ftype, @kind)"/>
              <xsl:text>, pointer :: a(</xsl:text>
              <xsl:call-template name="add-dimensions">
                <xsl:with-param name="rank" select="$rank"/>
              </xsl:call-template>
              <xsl:text>)
  integer, intent(in) :: lb(</xsl:text>
          <xsl:value-of select="concat($rank, '), ub(', $rank)"/> <xsl:text>)
  integer, intent(out), optional :: rc
  integer :: rank = </xsl:text> <xsl:value-of select="$rank"/> <xsl:text>

  call ARMCI_Malloc_</xsl:text> <xsl:value-of select="$type"/>
                    <xsl:text>(a, rank, lb, ub, rc)
  if (rc .ne. 0) print *, "ERROR ARMCI_Malloc_</xsl:text>
                    <xsl:value-of select="$ext"/> <xsl:text>: error = ", rc

end subroutine ARMCI_Malloc_</xsl:text>
                    <xsl:value-of select="$ext"/> <xsl:text>

</xsl:text>
    
  </xsl:template>


  <!--
    - defineFree
    -->
  <xsl:template name="defineFree">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="dim" select="concat($rank, 'D')"/>
    <xsl:variable name="ext" select="concat($rank, 'D', $type)"/>

    <xsl:value-of select="$comment-header"/>

<xsl:text>
subroutine ARMCI_Free_</xsl:text>
           <xsl:value-of select="$ext"/> <xsl:text>(a, rc)
  use F90_kinds
  implicit none
  interface
    subroutine ARMCI_Free_</xsl:text>
               <xsl:value-of select="$type"/> <xsl:text>(a, rank, rc)
      use F90_kinds
      </xsl:text> <xsl:value-of select="concat(@ftype, @kind)"/>
                  <xsl:text>, pointer :: a(</xsl:text>
                  <xsl:call-template name="add-dimensions">
                    <xsl:with-param name="rank" select="$rank"/>
                  </xsl:call-template>
                  <xsl:text>)
      integer, intent(in)  :: rank
      integer, intent(out) :: rc
    end subroutine ARMCI_Free_</xsl:text>
                  <xsl:value-of select="$type"/> <xsl:text>
  end interface
  </xsl:text> <xsl:value-of select="concat(@ftype, @kind)"/>
              <xsl:text>, pointer :: a(</xsl:text>
              <xsl:call-template name="add-dimensions">
                <xsl:with-param name="rank" select="$rank"/>
              </xsl:call-template>
              <xsl:text>)
  integer, intent(out), optional :: rc
  integer :: rank = </xsl:text> <xsl:value-of select="$rank"/> <xsl:text>

  call ARMCI_Free_</xsl:text> <xsl:value-of select="$type"/>
                  <xsl:text>(a, rank, rc)
  if (rc .ne. 0) print *, "ERROR ARMCI_Free_</xsl:text>
                  <xsl:value-of select="$ext"/> <xsl:text>: error = ", rc

end subroutine ARMCI_Free_</xsl:text>
                  <xsl:value-of select="$ext"/> <xsl:text>

</xsl:text>
    
  </xsl:template>


  <!--
    - add-dimensions
    -->
  <xsl:template name="add-dimensions">
    <xsl:param name="rank"      select="1"/>
    <xsl:choose>
      <xsl:when test="$rank > '1'">
        <xsl:text>:,</xsl:text>
        <xsl:call-template name="add-dimensions">
          <xsl:with-param name="rank" select="$rank - '1'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$rank = '1'">
        <xsl:text>:</xsl:text>
      </xsl:when>
    </xsl:choose>
  </xsl:template>


  <!--
    - add-allocate-info
    -->
  <xsl:template name="add-allocate-info">
    <xsl:param name="rank"      select="1"/>

    <xsl:choose>
      <xsl:when test="$rank > '1'">
        <xsl:call-template name="add-allocate-info">
          <xsl:with-param name="rank" select="$rank - '1'"/>
        </xsl:call-template>
      </xsl:when>
    </xsl:choose>

    <xsl:if test="$rank > '1'">
      <xsl:text>,</xsl:text>
    </xsl:if>
    <xsl:value-of select="concat( 'lb(', $rank, '):ub(', $rank, ')' )"/>

  </xsl:template>


  <!--
    - openArrayModule
    -->
  <xsl:template name="openArrayModule">
<xsl:text>module ARMCI
  use F90_kinds
</xsl:text>
  </xsl:template>


  <!--
    - closeArrayModule
    -->
  <xsl:template name="closeArrayModule">
<xsl:text>
end module ARMCI


</xsl:text>
  </xsl:template>


  <!--
    - openFile
    -->
  <xsl:template name="openFile">
<xsl:text>!! LANL:license
! -------------------------------------------------------------------------
! This SOFTWARE has been authored by an employee or employees of the
! University of California, operator of the Los Alamos National Laboratory
! under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
! The U.S. Government has rights to use, reproduce, and distribute this
! SOFTWARE.  The public may copy, distribute, prepare derivative works and
! publicly display this SOFTWARE without charge, provided that this Notice
! and any statement of authorship are reproduced on all copies.  Neither
! the Government nor the University makes any warranty, express or implied,
! or assumes any liability or responsibility for the use of this SOFTWARE.
! If SOFTWARE is modified to produce derivative works, such modified
! SOFTWARE should be clearly marked, so as not to confuse it with the
! version available from LANL.
! -------------------------------------------------------------------------
! LANL:license
! -------------------------------------------------------------------------
!
! ARMCI_Malloc_F.f90 - this file is automatically created
!
! The Fortran procedures in this file allocate and free ARMCI based arrays.
!
! -------------------------------------------------------------------------


</xsl:text>

  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile"/>

</xsl:stylesheet>
