<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - cpp-stubs.cc.xsl: creates a file of empty C++ stubs
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:import href="type-conversions.xsl"/>

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>


 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">

    <xsl:call-template name="openFile"/>
    <xsl:apply-templates select="scope"/>
    <xsl:call-template name="closeFile"/>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="scope">
     <xsl:apply-templates select="structure"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="structure">
     <xsl:apply-templates select="method"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="method">
    <xsl:variable name="chasm_rtn">
      <xsl:apply-templates select="return" mode="decl_call_cc_rtn"/>
    </xsl:variable>

    <xsl:if test="@kind='purevirtual'">
      <xsl:if test="@inlined = 'no'">
        <xsl:value-of select="$newline"/>
        <xsl:apply-templates select="return" mode="decl_cc_rtn"/>
        <xsl:text> </xsl:text>
        <xsl:if test="../../@kind = 'namespace'">
          <xsl:value-of select="../../@name"/>
          <xsl:text>::</xsl:text>
        </xsl:if>
        <xsl:value-of select="../@name"/>
        <xsl:text>::</xsl:text>

        <xsl:value-of select="@name"/>

        <xsl:text>(</xsl:text>
        <xsl:apply-templates select="arg" mode="decl_cc_arg"/>
	<xsl:text>)</xsl:text>
	
        <xsl:if test="exception">
          <xsl:value-of select="concat(' throw (', exception/@name, ')')"/>
        </xsl:if>

        <xsl:value-of select="concat($newline, '{', $newline)"/>

        <xsl:if test="$chasm_rtn != ''">
          <xsl:value-of select="concat($chasm_rtn, $newline)"/>
          <xsl:value-of select="concat('  return chasm_rtn;', $newline)"/>
        </xsl:if>

        <xsl:text>}</xsl:text>
        <xsl:value-of select="$newline"/>
      </xsl:if>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_cc_rtn">
    <xsl:call-template name="decl_cc_rtn"/>
  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="arg" mode="decl_cc_arg">

    <xsl:call-template name="decl_cc_arg"/>
    <xsl:text> </xsl:text>
    <xsl:value-of select="@name"/>

    <xsl:if test="position() != last()">
      <xsl:text>, </xsl:text>
    </xsl:if>

  </xsl:template>


  <!--
    - 
    -->
  <xsl:template match="return" mode="decl_call_cc_rtn">
    <xsl:variable name="chasm_rtn">
      <xsl:call-template name="decl_call_cc_rtn"/>
    </xsl:variable>

    <xsl:if test="$chasm_rtn != ''">
      <xsl:value-of select="concat('  ', $chasm_rtn, ' chasm_rtn')"/>
      <xsl:if test="type/@kind = 'ptr'">
	<xsl:text> = 0</xsl:text>
      </xsl:if>
      <xsl:text>;</xsl:text>
    </xsl:if>
  </xsl:template>


  <!--
    - openFile: print file header information
    -->
  <xsl:template name="openFile">

    <xsl:param name="file" select="'unknown_file'"/>
    <xsl:text>//</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>// this file is automatically generated</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>//</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="$newline"/>

    <xsl:value-of select="concat('#include &quot;', @name)"/>
    <xsl:value-of select="concat('&quot;', $newline, $newline)"/>

  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile">
    <xsl:value-of select="$newline"/>
  </xsl:template>


</xsl:stylesheet>
