<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - Templates for type conversions
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>

  <!--
    - declare cc (C to C) templates
    -->

  <xsl:template name="decl_cc_rtn">
    <xsl:choose>
      <xsl:when test="type/@kind = 'void'">
        <xsl:text>void</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'int'">
        <xsl:choose>
          <xsl:when test="type/@ikind = 'char'">
            <xsl:text>char</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uchar'">
            <xsl:text>unsigned char</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'short'">
            <xsl:text>short</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ushort'">
            <xsl:text>unsigned short</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'int'">
            <xsl:text>int</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uint'">
            <xsl:text>unsigned int</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'long'">
            <xsl:text>long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulong'">
            <xsl:text>unsigned long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'longlong'">
            <xsl:text>long long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulonglong'">
            <xsl:text>unsigned long long</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'float'">
        <xsl:choose>
          <xsl:when test="type/@fkind = 'float'">
            <xsl:text>float</xsl:text>
          </xsl:when>
          <xsl:when test="type/@fkind = 'dbl'">
            <xsl:text>double</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'bool'">
        <xsl:text>bool</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'enum'">
        <xsl:value-of select="type/@enumname"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'">
        <xsl:if test="type/@const = 'yes'">
          <xsl:text>const </xsl:text>
        </xsl:if>
        <xsl:value-of select="type/@usertype"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'tref'">
        <xsl:text>tref</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'ptr'">
        <xsl:for-each select="type/indirect">
          <xsl:call-template name="decl_cc_rtn"/>
        </xsl:for-each>
        <xsl:text>*</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="decl_cc_arg">
    <xsl:choose>
      <xsl:when test="type/@kind = 'int'">
        <xsl:choose>
          <xsl:when test="type/@ikind = 'char'">
            <xsl:text>char</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uchar'">
            <xsl:text>unsigned char</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'short'">
            <xsl:text>short</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ushort'">
            <xsl:text>unsigned short</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'int'">
            <xsl:text>int</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'uint'">
            <xsl:text>unsigned int</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'long'">
            <xsl:text>long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulong'">
            <xsl:text>unsigned long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'longlong'">
            <xsl:text>long long</xsl:text>
          </xsl:when>
          <xsl:when test="type/@ikind = 'ulonglong'">
            <xsl:text>unsigned long long</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'float'">
        <xsl:choose>
          <xsl:when test="type/@fkind = 'float'">
            <xsl:text>float</xsl:text>
          </xsl:when>
          <xsl:when test="type/@fkind = 'dbl'">
            <xsl:text>double</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>UNSUPPORTED</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="type/@kind = 'bool'">
        <xsl:text>bool</xsl:text>
      </xsl:when>
      <xsl:when test="type/@kind = 'enum'">
        <xsl:value-of select="type/@enumname"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'">
        <xsl:if test="type/@const = 'yes'">
          <xsl:text>const </xsl:text>
        </xsl:if>
        <xsl:value-of select="type/@usertype"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'ptr'">
        <xsl:for-each select="type/indirect">
          <xsl:call-template name="decl_cc_arg"/>
          <xsl:text>*</xsl:text>
        </xsl:for-each>
      </xsl:when>
      <xsl:when test="type/@kind = 'ref'">
        <xsl:for-each select="type/indirect">
          <xsl:call-template name="decl_cc_arg"/>
          <xsl:text>&amp;</xsl:text>
        </xsl:for-each>
      </xsl:when>
      <xsl:when test="type/@kind = 'tref'">
        <xsl:text>tref</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="decl_call_cc_rtn">
    <xsl:choose>
      <xsl:when test="type/@kind = 'void'"/>
      <xsl:when test="type/@kind = 'int'">
        <xsl:call-template name="decl_cc_rtn"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'float'">
        <xsl:call-template name="decl_cc_rtn"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'bool'">
        <xsl:call-template name="decl_cc_rtn"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'enum'">
        <xsl:value-of select="type/@enumname"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'usertype'">
        <xsl:if test="type/@const = 'yes'">
          <xsl:text>const </xsl:text>
        </xsl:if>
        <xsl:value-of select="type/@usertype"/>
      </xsl:when>
      <xsl:when test="type/@kind = 'ptr'">
        <xsl:for-each select="type/indirect">
          <xsl:call-template name="decl_cc_rtn"/>
          <xsl:text>*</xsl:text>
        </xsl:for-each>
      </xsl:when>
      <xsl:when test="type/@kind = 'tref'">
        <xsl:text>tref</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>UNSUPPORTED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


</xsl:stylesheet>
