<?xml version='1.0' encoding="ISO-8859-1" ?>

<!DOCTYPE html [
 <!ENTITY % characters SYSTEM "xhtml-lat1.ent">
 %characters;
]>

<!--
xsldoc.xsl : generates documentation for an XSLT stylesheet.
Author     : Fabrice Desr - DMI/GRI
Date       : 24/08/2001
Version    : 0.1
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output method="html"
            version="4.0"
	    encoding="ISO-8859-1"
	    indent="yes" />

<!--
showsource parameter. Valid values are :
'yes' : source code of each template will be included.
'no'  : no source code included.
-->
<xsl:param name="showsource" select="'no'"/>

<!--
filename parameter
No default value
-->
<xsl:param name="filename"/>

<!--
 Infos gnrales sur la feuille de style
-->	    
<xsl:template match="/xsl:stylesheet">
<html>
<head>
<title>XSL stylesheet documentation</title>
<link rel="stylesheet" type="text/css" href="xsltdoc.css"/>
</head>
<body>

<h1>Stylesheet auto-generated documentation
<xsl:if test="string-length($filename) > 0"> for <xsl:value-of select="$filename"/></xsl:if></h1>

<xsl:if test="(count(preceding-sibling::comment()[1]) > 0)">
<div class="subtoc">
<pre class="doc">
<xsl:for-each select="preceding-sibling::comment()[1]">
<xsl:value-of select="."/><br/>
</xsl:for-each>
</pre>
</div>
</xsl:if>

<xsl:if test="xsl:output/@method">
<h4>Ouput method : <xsl:value-of select="xsl:output/@method"/></h4>
</xsl:if>

<xsl:if test="count(xsl:include) > 0">
<div class="illegal-example">
This stylesheet contains &lt;xsl:include/&gt; elements : it won't be
correctly analysed.
</div>
</xsl:if>

<xsl:if test="count(xsl:import) > 0">
<div class="illegal-example">
This stylesheet contains &lt;xsl:import/&gt; elements : it won't be
correctly analysed.
</div>
</xsl:if>

<xsl:if test="count(xsl:param) > 0">
<h2>Global parameters</h2>
<xsl:call-template name="printParam"/>
</xsl:if>

<xsl:if test="count(xsl:variable) > 0">
<h2>Global variables</h2>
<xsl:call-template name="printVariable"/>
</xsl:if>

<xsl:if test="count(xsl:template) > 0">
<h2><xsl:value-of select="count(xsl:template)"/> Templates (
<xsl:value-of select="count(xsl:template[@name])"/> named, 
<xsl:value-of select="count(xsl:template[@match])"/> matched)</h2>
<table cellpadding="3" ><tr><th>#</th><th>name</th><th>match</th><th>mode</th></tr>
<xsl:for-each select="xsl:template">
<tr>
    <xsl:variable name="anchor">#<xsl:call-template name="buildAnchor"/></xsl:variable>
    <td><a href="{$anchor}"><xsl:value-of select="position()"/></a></td>
    <td>&nbsp;<xsl:value-of select="@name"/></td>
    <td>&nbsp;<xsl:value-of select="@match"/></td>
    <td>&nbsp;<xsl:value-of select="@mode"/></td>
</tr>
</xsl:for-each>
</table>
</xsl:if>
<hr/>

<xsl:apply-templates select="xsl:template"/>

<xsl:if test="count(xsl:template[@name]) > 0">
<xsl:call-template name="crossRef"/>
</xsl:if>

<table width="100%">
<tr><td>
<cite>Generated by <a href="xsldoc.tar.gz">xsldoc</a> &copy; 2001 <a href="mailto:fabrice.desre@francetelecom.com">Fabrice
Desr</a></cite>
</td>
</tr>
</table>
</body>
</html>
</xsl:template>

<!--
 Infos sur un template
-->
<xsl:template match="xsl:template">
<xsl:variable name="anchor"><xsl:call-template name="buildAnchor"/></xsl:variable>
<a name="{$anchor}"/>
<h2><xsl:call-template name="getName"/></h2>
<xsl:if test="(count(descendant::comment())>0) or (count(preceding-sibling::comment()[1]) > 0)">
<div class="subtoc">
<h3>Description</h3>
<pre class="doc">
<xsl:for-each select="preceding-sibling::comment()[1]|descendant::comment()">
<xsl:value-of select="."/><br/>
</xsl:for-each>
</pre>
</div>
</xsl:if>

<xsl:if test="count(xsl:param) > 0">
<h3>Parameters</h3>
<xsl:call-template name="printParam"/>
</xsl:if>

<xsl:if test="count(descendant::xsl:variable) > 0">
<h3>Variables used</h3>
<xsl:call-template name="printVariableDeep"/>
</xsl:if>

<xsl:if test="count(descendant::xsl:call-template) > 0">
<h3>Templates called</h3>
<table cellpadding="3" >
<tr><th>name</th><th>parameters</th></tr>
<xsl:for-each select="descendant::xsl:call-template">
<xsl:variable name="state" select="."/>
<xsl:variable name="anchor">#<xsl:call-template name="buildAnchor"/></xsl:variable>
<tr>
<td>&nbsp;<a href="{$anchor}"><xsl:value-of select="@name"/></a></td>
<xsl:variable name="tName" select="@name"/>
<td>&nbsp;
<xsl:for-each select="xsl:with-param">
<xsl:value-of select="@name"/><br/>
   <xsl:variable name="pName" select="@name"/>
<!-- vrification de l'usage des paramtres -->
<xsl:if test="count(//xsl:template[@name = $tName]/xsl:param[@name = $pName]) = 0">
<div class="illegal-example">
Unknow parameter in target template : <xsl:value-of select="$pName"/>
</div>
</xsl:if>
</xsl:for-each>

<!-- Warning si paramtres absents -->
<xsl:for-each select="//xsl:template[@name = $tName]/xsl:param">
<xsl:variable name="pName" select="@name"/>
<xsl:if test="count($state/xsl:with-param[@name = $pName]) = 0">
<div class="illegal-example">
<xsl:choose>
<xsl:when test="@select">
Warning : Missing parameter <xsl:value-of select="$pName"/> when calling target template (default value =
<xsl:value-of select="@select"/>)
</xsl:when>
<xsl:otherwise>
Warning : Missing parameter <xsl:value-of select="$pName"/> when calling target template (<strong>NO DEFAULT
VALUE</strong>)
</xsl:otherwise>
</xsl:choose>
</div>
</xsl:if>
</xsl:for-each>

</td>
</tr>
</xsl:for-each>
</table>
</xsl:if>

<xsl:if test="count(descendant::xsl:apply-templates) > 0">
<h3>Templates applied</h3>
<table cellpadding="3" >
<tr><th>select</th><th>parameters</th></tr>
<xsl:for-each select="descendant::xsl:apply-templates">
<xsl:variable name="anchor">#<xsl:call-template name="buildAnchor"/></xsl:variable>
<tr>
<td>&nbsp;<a href="{$anchor}"><xsl:value-of select="@select"/></a></td>
<td>&nbsp;<xsl:for-each select="xsl:with-param">
<xsl:value-of select="@name"/><br/>
</xsl:for-each></td>
</tr>
</xsl:for-each>
</table>
</xsl:if>

<xsl:if test="$showsource = 'yes'">
<xsl:call-template name="verbatim"/>
</xsl:if>

<hr/>
</xsl:template>

<!--
 Construit un tableau de cross rfrence : qui appelle qui ?
-->
<xsl:template name="crossRef">
<h2>Cross reference for named templates</h2>
<table cellpadding="3" >
<!-- Premire ligne : tous les templates -->
<tr><td align="center">Is called by &raquo;</td>
<xsl:for-each select="//xsl:template">
<td><xsl:call-template name="getNameShort"/></td>
</xsl:for-each>
</tr>
<!-- Pour chaque ligne, traitement d'un named template -->
<xsl:for-each select="//xsl:template[@name]">
<tr><td><xsl:value-of select="@name"/></td>
<xsl:variable name="tName" select="@name"/>
<xsl:for-each select="//xsl:template">
<td align="center"><xsl:value-of select="count(descendant::xsl:call-template[@name = $tName])"/></td>
</xsl:for-each>
</tr>
</xsl:for-each>
</table>
<!-- test : y a t il des templates inutiliss -->
<xsl:for-each select="//xsl:template[@name]">
<xsl:variable name="tName" select="@name"/>
<xsl:variable name="used" select="count(//xsl:template/descendant::xsl:call-template[@name = $tName])"/>
<xsl:if test="$used = 0">
<div class="illegal-example">
Named template <xsl:value-of select="$tName"/> is never called !
</div>
</xsl:if>
</xsl:for-each>

<hr/>

</xsl:template>

<!--
 Affiche les fils xsl:param du contexte courant
-->
<xsl:template name="printParam">
<table cellpadding="3"><tr ><th>name</th><th>default value</th><th>comment</th></tr>
<xsl:for-each select="xsl:param">
<tr><td>&nbsp;<xsl:value-of select="@name"/></td>
    <td>&nbsp;<xsl:value-of select="@select"/></td>
    <td>
    <pre class="doc"><xsl:for-each select="preceding-sibling::comment()[1]|descendant::comment()">
    <xsl:value-of select="."/></xsl:for-each></pre>
    </td>
</tr>
</xsl:for-each>
</table>
</xsl:template>

<!--
 Affiche les fils xsl:variable du contexte courant
-->
<xsl:template name="printVariable">
<table cellpadding="3" ><tr><th>name</th><th>default value</th><th>comment</th></tr>
<xsl:for-each select="xsl:variable">
<tr><td>&nbsp;<xsl:value-of select="@name"/></td>
    <td>&nbsp;<xsl:value-of select="@select"/></td>
    <td>
    <pre class="doc"><xsl:for-each select="preceding-sibling::comment()[1]|descendant::comment()">
    <xsl:value-of select="."/></xsl:for-each></pre>
    </td>
</tr>
</xsl:for-each>
</table>
</xsl:template>

<!--
 Affiche les descendants xsl:variable du contexte courant
-->
<xsl:template name="printVariableDeep">
<table cellpadding="3" ><tr><th>name</th><th>default value</th></tr>
<xsl:for-each select="descendant::xsl:variable">
<tr><td>&nbsp;<xsl:value-of select="@name"/></td>
    <td>&nbsp;<xsl:value-of select="@select"/></td>
</tr>
</xsl:for-each>
</table>
</xsl:template>

<!--
Construit une ancre HTML pour un template
-->
<xsl:template name="buildAnchor">
<xsl:value-of select="@name"/>_<xsl:value-of select="@match"/>_<xsl:value-of select="@mode"/>
</xsl:template>

<!--
 Reconstruit le nom d'un template
 -->
<xsl:template name="getName">
&lt;xsl:template <xsl:if test="@name">name=&quot;<xsl:value-of
select="@name"/>&quot;&nbsp;</xsl:if><xsl:if test="@match">match=&quot;<xsl:value-of
select="@match"/>&quot;&nbsp;</xsl:if><xsl:if test="@mode">mode=&quot;<xsl:value-of
select="@mode"/>&quot;</xsl:if>&gt; 
</xsl:template>

<!--
 Reconstruit le nom d'un template : version courte
 -->
<xsl:template name="getNameShort">
<xsl:if test="@name">name=&quot;<xsl:value-of
select="@name"/>&quot;&nbsp;</xsl:if><xsl:if test="@match">match=&quot;<xsl:value-of
select="@match"/>&quot;&nbsp;</xsl:if><xsl:if test="@mode">mode=&quot;<xsl:value-of
select="@mode"/>&quot;</xsl:if> 
</xsl:template>

<!--
 Affiche le source d'un template
-->
<xsl:template name="verbatim">
<div class="verbatim">
<h3>Template source</h3>
<pre>
<xsl:apply-templates select="*" mode="verbatim"/>
</pre>
</div>
</xsl:template>

<xsl:template match="*" mode="verbatim">
<xsl:text>&lt;</xsl:text>
<xsl:value-of select="name(.)"/>
<xsl:for-each select="@*">
<xsl:text> </xsl:text>
<xsl:value-of select="name(.)"/>
<xsl:text>="</xsl:text>
<xsl:value-of select="."/>
<xsl:text>"</xsl:text>
</xsl:for-each>
<xsl:choose>
<xsl:when test="*|text()">
<xsl:text>&gt;</xsl:text>
<xsl:apply-templates select="*|text()" mode="verbatim"/>
<xsl:for-each select="ancestor::*">
<xsl:text>  </xsl:text>
</xsl:for-each>
<xsl:text>&lt;/</xsl:text>
<xsl:value-of select="name(.)"/>
<xsl:text>&gt;</xsl:text>
</xsl:when>
<xsl:otherwise>
<xsl:text>/&gt;</xsl:text>
</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template match="text()" mode="verbatim">
<xsl:copy/>
</xsl:template>

   
</xsl:stylesheet>
