<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - F90CopyCreate.c.xsl: creates a file of C functions that call Fortran
  -   procedures to create and destroy F90 Arrays.  These procedures are
  -   used to allocate an array in Fortran, save the array descriptor in C,
  -   modify the memory in C, then pass the descriptor back to Fortran.
  -   WARNING, array memory must no be free'd in C.
  -
  - Output should be directed to the file, F90CopyDesc.c
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

  <xsl:variable name="comment-header">
    <xsl:text>/*-----------------------------------------------------------------------*/</xsl:text>
  </xsl:variable>


 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">
    <xsl:call-template name="openFile"/>

    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">

        <xsl:call-template name="define-copy-desc">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>

        <xsl:call-template name="define-dealloc">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>

      </xsl:for-each>
    </xsl:for-each>

    <xsl:call-template name="closeFile"/>
  </xsl:template>


  <!--
    - define-copy-desc
    -->
  <xsl:template name="define-copy-desc">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="full_name"
                  select="concat('F90_CopyDesc', $rank, 'D', $type)"/>

    <xsl:value-of select="concat($comment-header, $newline)"/>
    <xsl:value-of select="concat('void ', $full_name)"/>
    <xsl:value-of select="concat('(void** dest, void* src)', $newline)"/>

    <xsl:value-of select="concat('{', $newline)"/>
    <xsl:text>  *dest = createArrayDesc(src, 0, </xsl:text>
    <xsl:value-of select="concat($rank, ', F90_ArrayPointerInDerived);')"/>
    <xsl:value-of select="concat($newline, '}', $newline, $newline)"/>
    
  </xsl:template>


  <!--
    - define-dealloc
    -->
  <xsl:template name="define-dealloc">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="full_name"
                  select="concat('F90_Dealloc', $rank, 'D', $type)"/>

    <xsl:value-of select="concat($comment-header, $newline)"/>
    <xsl:value-of select="concat('void ', $full_name)"/>
<xsl:text>(void* ptr) { free(ptr); }


</xsl:text>
    
  </xsl:template>


  <!--
    - openFile
    -->
  <xsl:template name="openFile">

<xsl:text>/** LANL:license
 * -------------------------------------------------------------------------
 * This SOFTWARE has been authored by an employee or employees of the
 * University of California, operator of the Los Alamos National Laboratory
 * under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 * The U.S. Government has rights to use, reproduce, and distribute this
 * SOFTWARE.  The public may copy, distribute, prepare derivative works and
 * publicly display this SOFTWARE without charge, provided that this Notice
 * and any statement of authorship are reproduced on all copies.  Neither
 * the Government nor the University makes any warranty, express or implied,
 * or assumes any liability or responsibility for the use of this SOFTWARE.
 * If SOFTWARE is modified to produce derivative works, such modified
 * SOFTWARE should be clearly marked, so as not to confuse it with the
 * version available from LANL.
 * -------------------------------------------------------------------------
 * LANL:license
 * -------------------------------------------------------------------------
 *
 * F90CopyDesc.c - this file is automatically created
 *
 * The functions in this file make a copy of a Fortran array descriptor.
 * They are called by the corresponding F90_CreateArrayXXX procedure.
 *
 */

#include &lt;CompilerCharacteristics.h&gt;
#include &lt;F90CreateArray.h&gt;
#include &lt;stdlib.h&gt;

</xsl:text>

  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile"/>


  <!--
    - upper-case
    -->
  <xsl:template name="upper-case">
    <xsl:param name="symbol" select="unknown_symbol"/>

    <xsl:value-of select="translate($symbol,
                             'abcdefghijklmnopqrstuvwxyz',
                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:template>


  <!--
    - lower-case
    -->
  <xsl:template name="lower-case">
    <xsl:param name="symbol" select="unknown_symbol"/>

    <xsl:value-of select="translate($symbol,
                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
                             'abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:template>


</xsl:stylesheet>
