<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - F90CreateArray.xsl: creates F90_CreateArray/F90_DestroyArray procedures.
  -
  - Output should be directed to the file, F90CreateArray.f90
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

  <xsl:variable name="comment-header">
    <xsl:text>!--------------------------------------------------------------------------</xsl:text>
  </xsl:variable>


 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">
    <xsl:call-template name="openFile"/>

    <xsl:call-template name="openArrayModule"/>
    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="defineArrayType">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:call-template name="closeArrayModule"/>

    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>

      <xsl:for-each select="../type">

        <xsl:call-template name="defineCreate">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>

        <xsl:call-template name="defineDestroy">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>

      </xsl:for-each>

    </xsl:for-each>

    <xsl:call-template name="closeFile"/>
  </xsl:template>


  <!--
    - defineArrayType
    -->
  <xsl:template name="defineArrayType">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="name"
                  select="concat('F90_Array', $rank, 'D', $type, '_t')"/>

    <xsl:value-of select="concat($newline, '  type ', $name, $newline)"/>
    <xsl:value-of select="concat('    ', @ftype, @kind)"/>
    <xsl:text>, pointer :: data(</xsl:text>
    <xsl:call-template name="add-dimensions">
      <xsl:with-param name="rank" select="$rank"/>
    </xsl:call-template>
    <xsl:value-of select="concat(')', $newline)"/>
    <xsl:value-of select="concat('  end type ', $name, $newline)"/>
    
  </xsl:template>


  <!--
    - defineCreate
    -->
  <xsl:template name="defineCreate">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="name"
                  select="concat('F90_CreateArray', $rank, 'D', $type)"/>
    <xsl:variable name="copy-name"
                  select="concat('F90_CopyDesc', $rank, 'D', $type)"/>
    <xsl:variable name="atype-decl"
                  select="concat('type(F90_Array', $rank, 'D', $type, '_t)')"/>

    <xsl:value-of select="$comment-header"/>
    <xsl:text>
subroutine </xsl:text> <xsl:value-of select="$name"/> <xsl:text>(ptr, lb, ub)
  use F90_kinds
  use F90_ArrayTypes
  interface
    subroutine </xsl:text> <xsl:value-of select="$copy-name"/>
      <xsl:text>(dest, src)
      use F90_ArrayTypes
      </xsl:text> <xsl:value-of select="$atype-decl"/>
      <xsl:text>, intent(out) :: dest
      </xsl:text> <xsl:value-of select="$atype-decl"/>
      <xsl:text>, intent(in) :: src
    end subroutine </xsl:text> <xsl:value-of select="$copy-name"/> <xsl:text>
  end interface
  </xsl:text> <xsl:value-of select="$atype-decl"/>
  <xsl:text>, intent(out) :: ptr
  integer(kind=IKIND_LONG), intent(in) :: lb(</xsl:text>
  <xsl:value-of select="concat($rank, '), ub(', $rank, ')', $newline)"/>
  <xsl:value-of select="concat('  ',@ftype, @kind, ', pointer :: a(')"/>
  <xsl:call-template name="add-dimensions">
    <xsl:with-param name="rank" select="$rank"/>
  </xsl:call-template>
  <xsl:text>)
  </xsl:text>  <xsl:value-of select="$atype-decl"/>
  <xsl:text> :: a_t

  allocate( a(</xsl:text>
  <xsl:call-template name="add-allocate-info">
    <xsl:with-param name="rank" select="$rank"/>
  </xsl:call-template>
  <xsl:text>) )
  a_t%data => a
  call </xsl:text> <xsl:value-of select="$copy-name"/> <xsl:text>(ptr, a_t)
end subroutine </xsl:text> <xsl:value-of select="$name"/> <xsl:text>

</xsl:text>
    
  </xsl:template>


  <!--
    - defineDestroy
    -->
  <xsl:template name="defineDestroy">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="name"
                  select="concat('F90_DestroyArray', $rank, 'D', $type)"/>
    <xsl:variable name="dealloc-name"
                  select="concat('F90_Dealloc', $rank, 'D', $type)"/>
    <xsl:variable name="atype-decl"
                  select="concat('type(F90_Array', $rank, 'D', $type, '_t)')"/>

    <xsl:value-of select="$comment-header"/>
<xsl:text>
subroutine </xsl:text> <xsl:value-of select="$name"/> <xsl:text>(a_t)
  use F90_ArrayTypes
  interface
    subroutine </xsl:text> <xsl:value-of select="$dealloc-name"/>
      <xsl:text>(a_t)
      use F90_ArrayTypes
      </xsl:text> <xsl:value-of select="$atype-decl"/>
      <xsl:text>, intent(inout) :: a_t
    end subroutine </xsl:text> <xsl:value-of select="$dealloc-name"/><xsl:text>
  end interface
  </xsl:text> <xsl:value-of select="$atype-decl"/>
  <xsl:text>, intent(inout) :: a_t
  deallocate(a_t%data)
  call </xsl:text> <xsl:value-of select="$dealloc-name"/> <xsl:text>(a_t)
end subroutine </xsl:text> <xsl:value-of select="$name"/> <xsl:text>


</xsl:text>
    
  </xsl:template>


  <!--
    - add-dimensions
    -->
  <xsl:template name="add-dimensions">
    <xsl:param name="rank"      select="1"/>
    <xsl:choose>
      <xsl:when test="$rank > '1'">
        <xsl:text>:,</xsl:text>
        <xsl:call-template name="add-dimensions">
          <xsl:with-param name="rank" select="$rank - '1'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$rank = '1'">
        <xsl:text>:</xsl:text>
      </xsl:when>
    </xsl:choose>
  </xsl:template>


  <!--
    - add-allocate-info
    -->
  <xsl:template name="add-allocate-info">
    <xsl:param name="rank"      select="1"/>

    <xsl:choose>
      <xsl:when test="$rank > '1'">
        <xsl:call-template name="add-allocate-info">
          <xsl:with-param name="rank" select="$rank - '1'"/>
        </xsl:call-template>
      </xsl:when>
    </xsl:choose>

    <xsl:if test="$rank > '1'">
      <xsl:text>,</xsl:text>
    </xsl:if>
    <xsl:value-of select="concat( 'lb(', $rank, '):ub(', $rank, ')' )"/>

  </xsl:template>


  <!--
    - openArrayModule
    -->
  <xsl:template name="openArrayModule">
<xsl:text>module F90_ArrayTypes
  use F90_kinds
</xsl:text>
  </xsl:template>


  <!--
    - closeArrayModule
    -->
  <xsl:template name="closeArrayModule">
<xsl:text>
end module F90_ArrayTypes


</xsl:text>
  </xsl:template>


  <!--
    - openFile
    -->
  <xsl:template name="openFile">
<xsl:text>!! LANL:license
! -------------------------------------------------------------------------
! This SOFTWARE has been authored by an employee or employees of the
! University of California, operator of the Los Alamos National Laboratory
! under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
! The U.S. Government has rights to use, reproduce, and distribute this
! SOFTWARE.  The public may copy, distribute, prepare derivative works and
! publicly display this SOFTWARE without charge, provided that this Notice
! and any statement of authorship are reproduced on all copies.  Neither
! the Government nor the University makes any warranty, express or implied,
! or assumes any liability or responsibility for the use of this SOFTWARE.
! If SOFTWARE is modified to produce derivative works, such modified
! SOFTWARE should be clearly marked, so as not to confuse it with the
! version available from LANL.
! -------------------------------------------------------------------------
! LANL:license
! -------------------------------------------------------------------------
!
! F90CreateArray.f90 - this file is automatically created
!
! The Fortran procedures in this file create and destroy pointers to
! arrays.  Standard usage is to call these procedures from C++.  The
! large number of procedures are needed, since Fortran is now
! strongly typed and there is no casting mechanism.
!
! -------------------------------------------------------------------------


</xsl:text>

  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile"/>

</xsl:stylesheet>
