<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - F90CopyCreate.h.xsl: creates a C header file for Fortran procedures
  -   that create and destroy F90 Arrays.  These procedures can be used to
  -   allocate an array in Fortran (called from C), save the array descriptor
  -   in C, modify the memory in C, then pass the descriptor back to Fortran.
  -   WARNING, array memory must no be free'd in C; memory must be free'd
  -   by calling the associated F90_DestroyArray functions.
  -
  - Output should be directed to the file, F90CreateArray.h
  -->


<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>


 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">
    <xsl:call-template name="openFile"/>

    <xsl:text>#ifdef F90_SYM_CASE_LOWER</xsl:text>
    <xsl:value-of select="concat($newline, $newline)"/>
    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="symbol-lowercase">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:text>#endif /* F90_SYM_CASE_LOWER */</xsl:text>
    <xsl:value-of select="concat($newline, $newline, $newline)"/>

    <xsl:text>#ifdef F90_SYM_CASE_MIXED</xsl:text>
    <xsl:value-of select="concat($newline, $newline)"/>
    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="symbol-mixedcase">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:text>#endif /* F90_SYM_CASE_MIXED */</xsl:text>
    <xsl:value-of select="concat($newline, $newline, $newline)"/>

    <xsl:text>#ifdef F90_SYM_CASE_UPPER</xsl:text>
    <xsl:value-of select="concat($newline, $newline)"/>
    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="symbol-uppercase">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:text>#endif /* F90_SYM_CASE_UPPER */</xsl:text>
    <xsl:value-of select="concat($newline, $newline, $newline)"/>

    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="define-functions">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:value-of select="$newline"/>

    <xsl:call-template name="closeFile"/>
  </xsl:template>


  <!--
    - define-create
    -->
  <xsl:template name="define-functions">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>

    <xsl:value-of select="concat('void F90_CreateArray', $rank, 'D', $type)"/>
    <xsl:value-of select="concat(' (void*, long*, long*);', $newline)"/>

    <xsl:value-of select="concat('void F90_DestroyArray', $rank, 'D', $type)"/>
    <xsl:value-of select="concat('(void*);', $newline, $newline)"/>

  </xsl:template>


  <!--
    - symbol-lowercase
    -->
  <xsl:template name="symbol-lowercase">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="copy_name"
                  select="concat('F90_CopyDesc', $rank, 'D', $type)"/>
    <xsl:variable name="copy_lower">
      <xsl:call-template name="lower-case">
        <xsl:with-param name="symbol" select="$copy_name"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="create_name"
                  select="concat('F90_CreateArray', $rank, 'D', $type)"/>
    <xsl:variable name="create_lower">
      <xsl:call-template name="lower-case">
        <xsl:with-param name="symbol" select="$create_name"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="destroy_name"
                  select="concat('F90_DestroyArray', $rank, 'D', $type)"/>
    <xsl:variable name="destroy_lower">
      <xsl:call-template name="lower-case">
        <xsl:with-param name="symbol" select="$destroy_name"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="dealloc_name"
                  select="concat('F90_Dealloc', $rank, 'D', $type)"/>
    <xsl:variable name="dealloc_lower">
      <xsl:call-template name="lower-case">
        <xsl:with-param name="symbol" select="$dealloc_name"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:value-of select="concat('# define ', $copy_name,
                        '      F90_SYMBOL_COND( ', $copy_lower, ' )' )"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="concat('# define ', $create_name,
                           '   F90_SYMBOL_COND( ', $create_lower, ' )' )"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="concat('# define ', $destroy_name,
                            '  F90_SYMBOL_COND( ', $destroy_lower, ' )' )"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="concat('# define ', $dealloc_name,
                       '       F90_SYMBOL_COND( ', $dealloc_lower, ' )' )"/>
    <xsl:value-of select="concat($newline, $newline)"/>

  </xsl:template>


  <!--
    - symbol-mixedcase
    -->
  <xsl:template name="symbol-mixedcase">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="copy_name"
                  select="concat('F90_CopyDesc', $rank, 'D', $type)"/>
    <xsl:variable name="create_name"
                  select="concat('F90_CreateArray', $rank, 'D', $type)"/>
    <xsl:variable name="destroy_name"
                  select="concat('F90_DestroyArray', $rank, 'D', $type)"/>
    <xsl:variable name="dealloc_name"
                  select="concat('F90_Dealloc', $rank, 'D', $type)"/>

    <xsl:value-of select="concat('# define ', $copy_name,
                        '      F90_SYMBOL_COND( ', $copy_name, ' )' )"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="concat('# define ', $create_name,
                           '   F90_SYMBOL_COND( ', $create_name, ' )' )"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="concat('# define ', $destroy_name,
                            '  F90_SYMBOL_COND( ', $destroy_name, ' )' )"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="concat('# define ', $dealloc_name,
                       '       F90_SYMBOL_COND( ', $dealloc_name, ' )' )"/>
    <xsl:value-of select="concat($newline, $newline)"/>

  </xsl:template>


  <!--
    - symbol-uppercase
    -->
  <xsl:template name="symbol-uppercase">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="copy_name"
                  select="concat('F90_CopyDesc', $rank, 'D', $type)"/>
    <xsl:variable name="copy_upper">
      <xsl:call-template name="upper-case">
        <xsl:with-param name="symbol" select="$copy_name"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="create_name"
                  select="concat('F90_CreateArray', $rank, 'D', $type)"/>
    <xsl:variable name="create_upper">
      <xsl:call-template name="upper-case">
        <xsl:with-param name="symbol" select="$create_name"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="destroy_name"
                  select="concat('F90_DestroyArray', $rank, 'D', $type)"/>
    <xsl:variable name="destroy_upper">
      <xsl:call-template name="upper-case">
        <xsl:with-param name="symbol" select="$destroy_name"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="dealloc_name"
                  select="concat('F90_Dealloc', $rank, 'D', $type)"/>
    <xsl:variable name="dealloc_upper">
      <xsl:call-template name="upper-case">
        <xsl:with-param name="symbol" select="$dealloc_name"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:value-of select="concat('# define ', $copy_name,
                        '      F90_SYMBOL_COND( ', $copy_upper, ' )' )"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="concat('# define ', $create_name,
                           '   F90_SYMBOL_COND( ', $create_upper, ' )' )"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="concat('# define ', $destroy_name,
                            '  F90_SYMBOL_COND( ', $destroy_upper, ' )' )"/>
    <xsl:value-of select="$newline"/>
    <xsl:value-of select="concat('# define ', $dealloc_name,
                       '       F90_SYMBOL_COND( ', $dealloc_upper, ' )' )"/>
    <xsl:value-of select="concat($newline, $newline)"/>

  </xsl:template>


  <!--
    - openFile
    -->
  <xsl:template name="openFile">

<xsl:text>/** LANL:license
 * -------------------------------------------------------------------------
 * This SOFTWARE has been authored by an employee or employees of the
 * University of California, operator of the Los Alamos National Laboratory
 * under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 * The U.S. Government has rights to use, reproduce, and distribute this
 * SOFTWARE.  The public may copy, distribute, prepare derivative works and
 * publicly display this SOFTWARE without charge, provided that this Notice
 * and any statement of authorship are reproduced on all copies.  Neither
 * the Government nor the University makes any warranty, express or implied,
 * or assumes any liability or responsibility for the use of this SOFTWARE.
 * If SOFTWARE is modified to produce derivative works, such modified
 * SOFTWARE should be clearly marked, so as not to confuse it with the
 * version available from LANL.
 * -------------------------------------------------------------------------
 * LANL:license
 * -------------------------------------------------------------------------
 *
 * F90CreateArray.h - this file is automatically created
 *
 * The C functions declared in this file are implemented in Fortran and
 * are used to create and destroy F90 Arrays.  These procedures can be used
 * to allocate an array in Fortran (called from C), save the array descriptor
 * in C, modify the memory in C, then pass the descriptor back to Fortran.
 *
 * WARNING, array memory must not be free'd in C; memory must be free'd
 * by calling the associated F90DestroyArray functions.
 *
 * The large number of procedures are needed, since Fortran is
 * now strongly typed and there is no casting mechanism.
 *
 */

#ifndef _CHASM_CREATE_ARRAY_H_
#define _CHASM_CREATE_ARRAY_H_

#include &lt;F90Compiler.h&gt;


#ifdef F90_ABSOFT
#  include &lt;compilers/Absoft.h&gt;
#  define getArrayDescSize getArrayDescSize_Absoft
#  define createArrayDesc createArrayDesc_Absoft
#endif

#ifdef F90_ALPHA
#  include &lt;compilers/Alpha.h&gt;
#  define getArrayDescSize getArrayDescSize_Alpha
#  define createArrayDesc createArrayDesc_Alpha
#endif

#ifdef F90_CRAY
#  include &lt;compilers/Cray.h&gt;
#  define getArrayDescSize getArrayDescSize_Cray
#  define createArrayDesc createArrayDesc_Cray
#endif

#ifdef F90_GNU
#  include &lt;compilers/GNU.h&gt;
#  define getArrayDescSize getArrayDescSize_GNU
#  define createArrayDesc createArrayDesc_GNU
#endif

#ifdef F90_G95
#  include &lt;compilers/G95.h&gt;
#  define getArrayDescSize getArrayDescSize_G95
#  define createArrayDesc createArrayDesc_G95
#endif

#ifdef F90_IBMXL
#  include &lt;compilers/IBMXL.h&gt;
#  define getArrayDescSize getArrayDescSize_IBMXL
#  define createArrayDesc createArrayDesc_IBMXL
#endif

#ifdef F90_INTEL
#  include &lt;compilers/Intel.h&gt;
#  define getArrayDescSize getArrayDescSize_Intel
#  define createArrayDesc createArrayDesc_Intel
#endif

#ifdef F90_INTEL_7
#  include &lt;compilers/Intel_7.h&gt;
#  define getArrayDescSize getArrayDescSize_Intel_7
#  define createArrayDesc createArrayDesc_Intel_7
#endif

#ifdef F90_LAHEY
#  include &lt;compilers/Lahey.h&gt;
#  define getArrayDescSize getArrayDescSize_Lahey
#  define createArrayDesc createArrayDesc_Lahey
#endif

#ifdef F90_MIPSPRO
#  include &lt;compilers/MIPSpro.h&gt;
#  define getArrayDescSize getArrayDescSize_MIPSpro
#  define createArrayDesc createArrayDesc_MIPSpro
#endif

#ifdef F90_NAG
#  include &lt;compilers/NAG.h&gt;
#  define getArrayDescSize getArrayDescSize_NAG
#  define createArrayDesc createArrayDesc_NAG
#endif

#ifdef F90_PGI
#  include &lt;compilers/PGI.h&gt;
#  define getArrayDescSize getArrayDescSize_PGI
#  define createArrayDesc createArrayDesc_PGI
#endif

#ifdef F90_SUNWSPRO
#  include &lt;compilers/SUNWspro.h&gt;
#  define getArrayDescSize getArrayDescSize_SUNWspro
#  define createArrayDesc createArrayDesc_SUNWspro
#endif


</xsl:text>

  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile">
    <xsl:text>#endif /* _CHASM_CREATE_ARRAY_H_ */
</xsl:text>
  </xsl:template>


  <!--
    - upper-case
    -->
  <xsl:template name="upper-case">
    <xsl:param name="symbol" select="unknown_symbol"/>

    <xsl:value-of select="translate($symbol,
                             'abcdefghijklmnopqrstuvwxyz',
                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:template>


  <!--
    - lower-case
    -->
  <xsl:template name="lower-case">
    <xsl:param name="symbol" select="unknown_symbol"/>

    <xsl:value-of select="translate($symbol,
                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
                             'abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:template>


</xsl:stylesheet>
