<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - testcreate.c.xsl: creates test routines for F90_CreateArray and
  -    F90_DestroyArray procedures.
  -
  - Output should be directed to the file, testArrays.f90
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

  <xsl:variable name="comment-header">
    <xsl:text>!--------------------------------------------------------------------------</xsl:text>
  </xsl:variable>


 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">
    <xsl:call-template name="openFile"/>

    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="define-testArray">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
          <xsl:with-param name="ftype" select="@ftype"/>
          <xsl:with-param name="ctype" select="@ctype"/>
          <xsl:with-param name="kind" select="@kind"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>

    <xsl:call-template name="closeFile"/>
  </xsl:template>


  <!--
    - define-testArray
    -->
  <xsl:template name="define-testArray">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:param name="ftype"/>
    <xsl:param name="ctype"/>
    <xsl:param name="kind"/>
    <xsl:variable name="name" select="concat('testArray', $rank, 'D', $type)"/>
    <xsl:variable name="atype" select="concat('F90_Array',$rank,'D',$type,'_t')"/>

    <xsl:value-of select="concat($newline, $comment-header, $newline)"/>
    <xsl:value-of select="concat('integer function ', $name, '(a_t)')"/>
    <xsl:text>
  use F90_kinds
  use F90_ArrayTypes
  integer, dimension(7) :: lb = (/2, 3, 4, 5, 6, 7, 8/)
  integer, dimension(7) :: ub = (/5, 7, 9, 11, 13, 15, 17/)
  integer, dimension(7) :: shp = (/4, 5, 6, 7, 8, 9, 10/)
  integer, dimension(7) :: sz = (/4, 20, 120, 840, 6720, 60480, 604800/)
  integer :: i, shpa(</xsl:text>
    <xsl:value-of select="concat($rank, ')', $newline)"/>
    <xsl:value-of select="concat('  type(', $atype, '), intent(in) :: a_t')"/>
    <xsl:value-of select="concat($newline, '  ', $name,' = 0')"/>

    <xsl:text>

  if (.not. associated(a_t%data)) then
    </xsl:text> <xsl:value-of select="$name"/> <xsl:text> = 1
    return
  end if

</xsl:text>

    <xsl:if test="$ctype">
      <xsl:text>  if (any(a_t%data .ne. -</xsl:text>
      <xsl:value-of select="$rank"/>
      <xsl:text>)) then
    print *, "ERROR </xsl:text>
      <xsl:value-of select="$name"/>
      <xsl:text>: any(a_t%data .ne. rank)"
    </xsl:text>
      <xsl:value-of select="$name"/>
      <xsl:text> = 2
    return
  end if

</xsl:text>
    </xsl:if>

    <xsl:text>  if (size(a_t%data) .ne. sz(</xsl:text>
    <xsl:value-of select="$rank"/>
    <xsl:text>)) then
    print *, "ERROR </xsl:text>
    <xsl:value-of select="$name"/>
    <xsl:text>: count =", sz(</xsl:text>
    <xsl:value-of select="$rank"/>
    <xsl:text>), " size =", size(a_t%data)
    </xsl:text>
    <xsl:value-of select="$name"/>
    <xsl:text> = 3
    return
  end if

  shpa = shape(a_t%data)
  do i = 1, </xsl:text>
    <xsl:value-of select="$rank"/>
    <xsl:text>
    if (shpa(i) .ne. shp(i)) then
      print *, "ERROR </xsl:text>
    <xsl:value-of select="$name"/>
    <xsl:text>: shape(a) =", shape(a_t%data), " shp =", shp
      </xsl:text>
    <xsl:value-of select="$name"/>
      <xsl:text> = 4
      return
    end if
    if (lbound(a_t%data,i) .ne. lb(i) .or. ubound(a_t%data,i) .ne. ub(i)) then
      print *, "ERROR </xsl:text>
    <xsl:value-of select="$name"/>
      <xsl:text>: i =", i, ": lb(i) =", lbound(a_t%data,i), </xsl:text>
      <xsl:text>" ub(i) =", ubound(a_t%data,i)
      </xsl:text>
    <xsl:value-of select="$name"/>
    <xsl:text> = 5
      return
    end if
  end do

end function </xsl:text>
    <xsl:value-of select="concat($name, $newline, $newline)"/>

  </xsl:template>


  <!--
    - openFile
    -->
  <xsl:template name="openFile">

<xsl:text>!! LANL:license
! -------------------------------------------------------------------------
! This SOFTWARE has been authored by an employee or employees of the
! University of California, operator of the Los Alamos National Laboratory
! under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
! The U.S. Government has rights to use, reproduce, and distribute this
! SOFTWARE.  The public may copy, distribute, prepare derivative works and
! publicly display this SOFTWARE without charge, provided that this Notice
! and any statement of authorship are reproduced on all copies.  Neither
! the Government nor the University makes any warranty, express or implied,
! or assumes any liability or responsibility for the use of this SOFTWARE.
! If SOFTWARE is modified to produce derivative works, such modified
! SOFTWARE should be clearly marked, so as not to confuse it with the
! version available from LANL.
! -------------------------------------------------------------------------
! LANL:license
! -------------------------------------------------------------------------
!
! testArrays.f90 - this file is automatically created
!
! This file tests the F90_CreateArray/F90_DestroyArray functions in libchasm.a.
!
!!

</xsl:text>

  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile"/>


  <!--
    - add-dimensions
    -->
  <xsl:template name="add-dimensions">
    <xsl:param name="rank"      select="1"/>
    <xsl:choose>
      <xsl:when test="$rank > '1'">
        <xsl:text>:,</xsl:text>
        <xsl:call-template name="add-dimensions">
          <xsl:with-param name="rank" select="$rank - '1'"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$rank = '1'">
        <xsl:text>:</xsl:text>
      </xsl:when>
    </xsl:choose>
  </xsl:template>


</xsl:stylesheet>
