module babel_t
 type SIDL_double_2d
    sequence
    integer (selected_int_kind(18)) :: d_array
    real (selected_real_kind(15, 307)), pointer, &
      dimension(:,:) :: d_data
  end type SIDL_double_2d
end module babel_t

module babel

  use babel_t

  interface createRow
    module procedure createRow2_p
  end interface

  interface get
     module procedure getg2_p
  end interface

  interface set
    module procedure setg2_p
  end interface

contains

  ! 
  ! The size of lower determines the dimension of the
  ! array.
  ! 
  subroutine createRow2_p(lower, upper, a_t)
    external SIDL_double__array_createRow_m

    real (selected_real_kind(15, 307)), target, dimension(1,1) :: a
    integer (selected_int_kind(9)), dimension(2), intent(in) :: lower, upper
    type(SIDL_double_2d), intent(out) :: a_t

    !
    ! Setting the pointer here forces the compiler to set the
    ! array descriptor in a_t%d_data.  This allows resetArrayDesc
    ! to be used rather than setArrayDesc; a more conservative
    ! approach.
    !
    a_t%d_data => a
    call SIDL_double__array_createRow_m(2, lower, upper, a_t)

  end subroutine createRow2_p

 subroutine getg2_p(a_t, index, value)
    type(SIDL_double_2d), intent(in) :: a_t
    integer (selected_int_kind(9)), intent(in), dimension(2) :: index
    real (selected_real_kind(15, 307)), intent(out) :: value

    value = a_t%d_data(index(1), index(2))
!    call SIDL_double__array_get_m(a_t, index, value)

  end subroutine getg2_p

  subroutine setg2_p(a_t, index, value)
    type(SIDL_double_2d) :: a_t
    integer (selected_int_kind(9)), intent(in), dimension(2) :: index
    real (selected_real_kind(15, 307)), intent(in) :: value

    a_t%d_data(index(1), index(2)) = value
!    call SIDL_double__array_set_m(a_t, index, value)

  end subroutine setg2_p


end module babel


program test_babel_F
  use babel
  implicit none

  type(SIDL_double_2d) :: a_t
  real (selected_real_kind(15, 307)) :: d

  integer, dimension(2) :: ashape, index
  integer, dimension(2) :: shp = (/4,3/)
  integer, dimension(2) :: albound = (/-1,-1/)
  integer, dimension(2) :: aubound = (/2,1/)

  integer :: i, j
  logical :: debug = (.false.), error = (.false.)

  call createRow(albound, aubound, a_t)

  ashape  = shape(a_t%d_data)
  albound = lbound(a_t%d_data)
  aubound = ubound(a_t%d_data)
  
   if (.not. associated(a_t%d_data)) then
      error = .true.
      print *, "ERROR, a_t%d_data is unassociated"
   end if

  if (debug) then
    print *, " size   of a_t%d_data =", size(a_t%d_data)
    print *, " lbound of a_t%d_data =", albound
    print *, " ubound of a_t%d_data =", aubound
    print *, " shape  of a_t%d_data =", shape(a_t%d_data)
    print *, '.'
    print '(3f8.2)', ((a_t%d_data(i,j),j=-1,1),i=-1,2)
    print *, '.'
  end if

  do i = 1, 2
    if (ashape(i) .ne. shp(i)) then
       error = .true.
       print *, "ERROR, shape(", i, ") = ", ashape(i)
    end if
    if (albound(i) .ne. -1) then
       error = .true.
       print *, "ERROR, lbound(", i, ") = ", albound(i)
    end if
    if (aubound(i) .ne. ashape(i) - 2) then
       error = .true.
       print *, "ERROR, ubound(", i, ") = ", aubound(i)
    end if
  end do

  if (size(a_t%d_data) .ne. 4*3) then
    error = .true.
    print *, "ERROR, size(a_t%d_data) = ", size(a_t%d_data)
  end if

  d = 1.0d0
  do i = -1, 2
    do j = -1, 1
       if (a_t%d_data(i,j) .ne. d) then
          error = .true.
          print *, "ERROR a(", i, j, ") = ",  a_t%d_data(i,j)
       end if
       d = d + 1.0d0
    end do
  end do

  index(1) = 0
  index(2) = 0
  call get(a_t, index, d)
  if (d .ne. 5.0) then
    error = .true.
    print *, "ERROR a(0,0) = ", d
  end if

  d = 3.3d0
  call set(a_t, index, d)
  call get(a_t, index, d)
  if (d .ne. 3.3d0) then
    error = .true.
    print *, "ERROR a(0,0) = ", d
  end if

  if (error) then
    print *, "FAILED create array correctly"
  end if

end program test_babel_F
